<?php
/**
 * YJSNLog Permits logging JavaScript errors, traces ans warnings to the server.
 *
 * The widget provides the following JavaScript methods to log:
 * <code><pre>
 * JL().debug(logObject) Method
 * JL().error Method
 * JL().fatal Method
 * JL().fatalException(logObject, e) Method
 * JL().info(logObject) Method
 * JL().log(level, logObject) Method
 * JL().setOptions Method
 * JL().trace(logObject) Method
 * JL().warn(logObject) Method
 * </pre></code>
 *
 * Setup using:
 * <code><pre>
 *         Yii::import('ext.YJSNLog.*');
 *        $this->widget('YJSNLog',array(
 *               'ajaxAppenderOptions'=>array(
 *                  'url'=>Yii::app()->createUrl('logger/jsn'),
 *                  'bufferSize'=>20,
 *                  'storeInBufferLevel':new CJavaScriptExpression('JL.getDebugLevel()'),
 *                  'level':new CJavaScriptExpression('JL.getErrorLevel()'),
 *                  'sendWithBufferLevel':new CJavaScriptExpression('JL.getErrorLevel()'),
 *           ));
 * </pre></code>
 * 
 * And create "LoggerController.php" file in the controllers directory with the following php code:
 * <pre><code>
 * Yii::import('ext.YJSNLog.LoggerController',true);
 * </pre></code>
 * 
 * Use like this (to avoid error when not loaded).<br/>
 *    <code>if(JL!==undefined){JL().<*method*>}</code><br/>
 *    Example: <code>if(JL!==undefined){JL().warn("message");}</code>
 *    
 * This is an example static utility method (in PHP) to enable JSNLog with
 * some project specific options to avoid unwanted reports from external
 * sources (including some SPAMware).
 * <pre></code>
 *   public static function setupJavaScriptDebug($reportUrl='logger/jsn') {
 *   	Yii::import('ext.YJSNLog.*');
 *   	Yii::app()->getController()
 *   		->widget('YJSNLog',array(
 *   			'handleOnError'=>true,
 *   			'ajaxAppenderOptions'=>array(
 *   					'url'=>Yii::app()->createUrl($reportUrl),
 *   					'disallow'=>implode(
 *   							'|',
 *   							array(
 *   									'"url":"https://maps.gstatic.com/maps-api-v3'
 *   									,'at Zba'
 *   									,'document.querySelector..title...textContent'
 *   									,"'removeChild'"
 *   									,'//tags.bkrtx.com/js/bk-coretag.js'
 *   									,'"msg":"Exception!","errorMsg":"Script error.","url":"","line number":0'
 *   							)
 *   					)
 *   			),
 *   	));
 *  }
 *  </code></pre>
 */
class YJSNLog extends CWidget
{
    /**
     * Global options
     * @property JSNGlobalOptions
     */
    public $globalOptions;

    /**
     * Options for ajax
     *
     * @property JSNAppenderOptions
     */
    public $ajaxAppenderOptions;

    /**
     * Options for Console
     * @property JSNAppenderOptions
     */
    public $consoleAppenderOptions;

    /**
     * Handle onError.
     */
    public $handleOnError=false;
    /**
     * When true, cache the code. Default: true
     */
    public $cache=false;

    /**
     * Initializes everything
     *
     * @return void
     */
    public function init()
    {
        parent::init();
        if($this->handleOnError) {
            $defaultAppenderOptions=array(
                'storeInBufferLevel'=>new CJavaScriptExpression('JL.getDebugLevel()'),
                'level'=>new CJavaScriptExpression('JL.getErrorLevel()'),
                'sendWithBufferLevel'=>new CJavaScriptExpression('JL.getErrorLevel()'),
            );
        } else {
            $defaultAppenderOptions=array();
        }

        if(is_array($this->ajaxAppenderOptions)) {
            $tmp=new JSNAppenderOptions();
            $tmp->setAttributes(array_merge($defaultAppenderOptions,$this->ajaxAppenderOptions),false);
            $this->ajaxAppenderOptions=$tmp;
        }
        if(is_array($this->consoleAppenderOptions)) {
            $tmp=new JSNAppenderOptions();
            $tmp->setAttributes(array_merge($defaultAppenderOptions,$this->consoleAppenderOptions),false);
            $this->consoleAppenderOptions=$tmp;
        }
        if(is_array($this->globalOptions)) {
            $tmp=new JSNGlobalOptions();
            $tmp->setAttributes($this->globalOptions,false);
            $this->globalOptions=$tmp;
        }
        $this->registerScripts();
    }

    public function run() {

    }

    /**
     * Registers the JS and CSS Files
     *
     * @return void
     */
    protected function registerScripts()
    {
        /* @var $am CAssetManager */
        $am=Yii::app()->assetManager;
        $path=$am->publish(
                __DIR__ . DIRECTORY_SEPARATOR . "assets");
        $cs=Yii::app()->clientScript;
        $cacheid=__CLASS__.$this->id;
        $cssFiles=array();
        $jsFiles=array('json3.min.js','jsnlog.min.js','jsnprune.js');
        foreach($cssFiles as $f) {
            $cs->registerCssFile("$path/css/$f");
        }
        foreach($jsFiles as $j) {
            $cs->registerScriptFile("$path/js/$j");
        }
        $js=false;
//         if($this->cache) {
//             $js=Yii::app()->cache->get($cacheid);
//         }
        if($js === false) {
            $js="";
            $appenders=array();
            if(isset($this->ajaxAppenderOptions)) {
                $jsaOptions=CJavaScript::encode($this->ajaxAppenderOptions->getOptions());
                $js.="var ajaxAppender=JL.createAjaxAppender('ajaxAppender');ajaxAppender.setOptions($jsaOptions);";
                $appenders[]=new CJavaScriptExpression('ajaxAppender');
            }
            if(isset($this->consoleAppenderOptions)) {
                $jscOptions=CJavaScript::encode($this->consoleAppenderOptions->getOptions());
                $js.="var consoleAppender=JL.createConsoleAppender('consoleAppender');consoleAppender.setOptions($jscOptions);";
                $appenders[]=new CJavaScriptExpression('consoleAppender');
            }
            $options=compact('appenders');
            if(isset($this->globalOptions)) {
                $options=array_merge($options,$this->globalOptions->getOptions());
            }
            $jsOptions=CJavaScript::encode($options);
            $js.="JL().setOptions($jsOptions);";
            if($this->handleOnError) {
                $js.=<<<EOJS
window.onerror = function (errorMsg, url, lineNumber, column, errorObj) {
    JL("onerrorLogger").fatalException({
        "msg": "Exception!",
        "errorMsg": errorMsg, "url": url,
        "line number": lineNumber, "column": column
    }, JSON.pruned(errorObj));

    // Tell browser to run its own error handler as well
    return false;
}
EOJS;
            }
            $js="(function(){ $js})();";
            //Yii::app()->cache->set($cacheid, $js, $this->cacheexpire);
        }
        $cs->registerScript($cacheid,$js,CClientScript::POS_HEAD);

   }
}
